import subprocess
import os
import re
import json
import shutil
import tempfile
from shutil import which
from pygustus.options.aug_options import *
from pkg_resources import resource_filename
import pygustus.fasta_methods as fm
import pygustus.gff_methods as gff
from concurrent.futures import ThreadPoolExecutor


def execute_bin_parallel(cmd, aug_options, jobs, chunksize, overlap, partition_sequences, part_hints, minsize, max_seq_size, debug_dir):
    print(f'Execute AUGUSTUS with {jobs} jobs in parallel.')

    input_file = aug_options.get_input_filename()[1]
    joined_outfile = aug_options.get_value_or_none('outfile')
    if not joined_outfile:
        joined_outfile = 'augustus.gff'

    options = list()
    outfiles = list()
    with tempfile.TemporaryDirectory(prefix='.tmp_') as tmpdir:
        hintsfile = aug_options.get_value_or_none('hintsfile')

        run_information = fm.split(
            input_file, tmpdir, chunksize, overlap, partition_sequences, minsize, max_seq_size)

        for ri in run_information:
            runno = str(ri['run'])
            fileidx = str(ri['fileidx'])
            seqinfo = ri['seqinfo']
            outfile = os.path.join(tmpdir, f'augustus_{runno}.gff')
            outfiles.append(outfile)
            curfile = create_split_filenanme(input_file, tmpdir, fileidx)
            aug_options.set_input_filename(curfile)
            aug_options.set_value('outfile', outfile)
            aug_options.remove('predictionStart')
            aug_options.remove('predictionEnd')
            if len(seqinfo) == 1 and list(seqinfo.values())[0][0] > 0 and list(seqinfo.values())[0][1] > 0:
                aug_options.set_value(
                    'predictionStart', list(seqinfo.values())[0][0])
                aug_options.set_value(
                    'predictionEnd', list(seqinfo.values())[0][1])
            if hintsfile and part_hints:
                tmp_hintsfile = os.path.join(
                    tmpdir, f'augustus_hints_{str(runno)}.gff')
                gff.create_hint_parts(
                    hintsfile, tmp_hintsfile, seqinfo)
                aug_options.set_value('hintsfile', tmp_hintsfile)
            options.append(aug_options.get_options())

        with ThreadPoolExecutor(max_workers=int(jobs)) as executor:
            for opt in options:
                executor.submit(execute_bin, cmd, opt)

        gff.join_aug_pred(joined_outfile, outfiles)
        print(f'Joined output written to: {joined_outfile}')

        if debug_dir:
            rmtree_if_exists(debug_dir, even_none_empty=True)
            shutil.copytree(src=tmpdir, dst=debug_dir)
            cmd_filename = os.path.join(debug_dir, 'aug_cmd_lines.txt')
            with open(cmd_filename, "w") as file:
                for o in options:
                    file.write(str(o) + '\n' + '\n')


def execute_bin(cmd, options):
    # execute given binary with given options
    result = ''

    try:
        result = subprocess.check_output(
            [cmd] + options,
            stderr=subprocess.STDOUT,
            universal_newlines=True)
    except subprocess.CalledProcessError as cpe:
        print("Returncode", cpe.returncode, cpe.output)

    if len(result.strip()):
        print(result.strip())


def check_bin(bin):
    if which(bin) is None:
        raise RuntimeError(
            f'{bin} cannot be found or is not executable!')


def check_file(inputfile):
    if not os.path.exists(inputfile):
        raise ValueError(f'Could not open {inputfile}')


def mkdir_if_not_exists(dir):
    if not os.path.exists(dir):
        os.makedirs(dir, exist_ok=True)


def rmtree_if_exists(dir, even_none_empty=False):
    if os.path.exists(dir):
        if even_none_empty or len(os.listdir(dir)) == 0:
            shutil.rmtree(dir)


def get_options(*args, options, path_to_params, program, **kwargs):
    if options is None:
        options = AugustusOptions(
            *args, parameter_file=path_to_params, app=program, **kwargs)
    else:
        for arg in args:
            options.add_argument(arg)
        for option, value in kwargs.items():
            options.set_value(option, value)
    return options


def get_path_to_binary(options, program):
    bin = options.get_value_or_none('path_to_bin')
    if bin and not os.path.exists(bin):
        raise ValueError(
            f'{program} binaries cannot be found under specified path: {bin}.')
    return bin


def get_config_item(name):
    config_file = resource_filename('pygustus', 'config.json')
    with open(config_file, 'r') as file:
        config = json.load(file)

    return config.get(name)


def set_config_item(name, value):
    config_file = resource_filename('pygustus', 'config.json')
    with open(config_file, 'r+') as file:
        config = json.load(file)
        config.update({name: value})
        file.seek(0)
        file.truncate()
        json.dump(config, file, indent=4, sort_keys=False)


def create_split_filenanme(inputfile, outputdir, idx):
    filename = os.path.basename(inputfile)
    f_name, f_ext = os.path.splitext(filename)
    s_filename = f'{f_name}.split.{str(idx)}{f_ext}'
    return os.path.join(outputdir, s_filename)


def check_aug_version(aug_bin, min_aug_version):
    result = subprocess.run(
        [aug_bin, '--version'],
        capture_output=True, encoding='UTF-8')

    version_str = re.findall(r'\d+.\d+.\d+', result.stderr)[0]
    min_version_no = version_str_to_int(min_aug_version)
    version_no = version_str_to_int(version_str)

    if version_no < min_version_no:
        raise RuntimeError(
            f'AUGUSTUS version {min_aug_version} or higher is required!')


def version_str_to_int(version_str):
    vnumbers = [int(x) for x in version_str.split('.')]
    vnumbers.reverse()
    version_no = sum(x * (100 ** i) for i, x in enumerate(vnumbers))
    return version_no



"""
A python wrapper for the training of AUGUSTUS.
"""

from pkg_resources import resource_filename

from pygustus.options.aug_options import *
import pygustus.util as util

__all__ = ['train', 'config_get_bin',
           'config_set_bin', 'config_set_default_bin']


PARAMETER_FILE = resource_filename('pygustus.options', 'parameters.json')


def train(*args, options=None, **kwargs):
    """
    Executes etraining and passes the given parameters as command line arguments.

    Args:
        *args (tuple): Only the the queryfilename should be passed here.
        options (AugustusOptions): Optional; If an instance of
            AugustusOptions is passed, it will be used for the call.
            Otherwise, a new instance is created based on the passed arguments
            (the default is None).
        **kwargs (dict): Arguments for Etraining or Pygustus: lists with
            possible parameters can be obtained from the help methods or 
            the Pygustus README (only Pygustus parameters).
    """

    pygustus_options = util.get_options(
        *args, options=options, path_to_params=PARAMETER_FILE, program='pygustus', **kwargs)

    etraining_command = config_get_bin()
    tmp_path_to_bin = util.get_path_to_binary(pygustus_options, 'ETRAINING')
    if tmp_path_to_bin:
        etraining_command = tmp_path_to_bin

    util.check_bin(etraining_command)

    aug_options = util.get_options(
        *args, options=options, path_to_params=PARAMETER_FILE, program='etraining', **kwargs)

    # check input file
    is_file, input_file = aug_options.get_input_filename()
    if is_file:
        if input_file:
            util.check_file(input_file)
        else:
            raise ValueError(f'Input file not specified.')

    util.execute_bin(etraining_command, aug_options.get_options())


def config_get_bin():
    """Outputs currently configured path to the executable of etraining.

    Returns:
        string: The currently configured path to the executable of etraining.
    """
    return util.get_config_item('etraining_bin')


def config_set_bin(value):
    """ Updates the configured path to the executable of etraining.

    Args:
        value (string): The path to the execuatble of etraining.

    Raises:
        RuntimeError: If the given path does not exist or the file
        is not executable.
    """
    util.check_bin(value)
    util.set_config_item('etraining_bin', value)


def config_set_default_bin():
    """Sets the configured path to the etraining executable to 'etraining'.

    This executable should exist if AUGUSTUS is properly installed
    on the system.
    """
    util.set_config_item('etraining_bin', 'etraining')



import json
from distutils.util import strtobool


class AugustusOption:

    def __init__(self, name, type, possible_values, description, usage, default_value, development, exclude):
        self.name = name
        self.development = development
        self.type = type
        self.possible_values = possible_values
        self.description = description
        self.usage = usage
        self.default_value = default_value
        self.exclude = exclude
        self.value = None

    def set_value(self, value, check=True):
        if check:
            self.check_value(value)
        self.value = value

    def check_value(self, value):
        # TODO: check given value according to option properties
        self.check_values(value)
        self.check_type(value)

    def check_values(self, value):
        # TODO: check if property is iterable
        if not self.possible_values is None:
            if not value in self.possible_values:
                raise ValueError(
                    f'Invalid value for parameter --{self.name}! Expected {self.possible_values}.')

    def check_type(self, value):
        check_fail = False

        if self.type == 'string' and not isinstance(value, str):
            check_fail = True

        if self.type == 'int' and not isinstance(value, int):
            check_fail = True

        if self.type == 'float' and not isinstance(value, float):
            check_fail = True

        if self.type == 'bool' and not isinstance(value, bool):
            check_fail = True

        if self.type == 'list<string>' and not isinstance(value, list):
            check_fail = True
        elif self.type == 'list<string>' and isinstance(value, list):
            if not value:
                raise ValueError(
                    f'Empty list for parameter --{self.name} is not supported! Expected {self.type}.')

            for s in value:
                if not isinstance(s, str):
                    check_fail = True

        if check_fail:
            raise ValueError(
                f'Invalid value type for parameter --{self.name}! Expected {self.type}.')

    def get_name(self):
        return self.name

    def get_exclude(self):
        if self.exclude:
            return self.exclude
        else:
            return []


class AugustusOptions:
    _allowed_options = {}

    def __init__(self, *args, parameter_file, app, **kwargs):
        self._parameter_file = parameter_file
        self._options = {}
        self._app = app
        self._args = []
        self.load_options()
        if len(args) > 0:
            self._args += args
        for option, value in kwargs.items():
            self.set_value(option, value)
        self.setPygustusDefaultValues()

    def add_arguments(self, *args):
        self._args += args

    def add_argument(self, arg):
        self._args.append(arg)

    def get_arguments(self):
        return self._args

    def set_value(self, option_name, value):
        if option_name not in self._allowed_options.keys():
            raise ValueError(
                'Invalid Parameter for Augustus: %s' % option_name)

        option = self._allowed_options[option_name]

        if self._app == 'pygustus':
            if 'augustus' in option.get_exclude() and 'etraining' in option.get_exclude():
                option.set_value(value)
                self._options[option_name] = option.value
        else:
            if not self._app in option.get_exclude():
                option.set_value(value)
                self._options[option_name] = option.value

    def get_value(self, option):
        if option not in self._options:
            raise ValueError('Unknown option: %s' % option)
        return self._options[option]

    def get_value_or_none(self, name):
        return self._options.get(name)

    def remove(self, name):
        if name in self._options:
            del self._options[name]

    def get_options(self):
        opts = []
        for option, value in self._options.items():
            if isinstance(value, list):
                opts.append("--%s=%s" % (option, ','.join(value)))
            else:
                opts.append("--%s=%s" % (option, value))
        opts += self._args
        return opts

    def __str__(self):
        optstr = ""
        for option, value in self._options.items():
            optstr += "--%s=%s " % (option, value)
        if len(self._args):
            optstr += " ".join(self._args)
        return optstr

    def load_options(self):
        self._allowed_options = load_allowed_options(self._parameter_file)

    def get_input_filename(self):
        if len(self._args) == 1:
            if (self._args[0].startswith('--')):
                return False, None
            else:
                return True, self._args[0]
        else:
            return False, None

    def set_input_filename(self, name):
        if len(self._args) == 1:
            self._args[0] = name
        else:
            # TODO: throw an error?
            print('Could not set filename!')

    def setPygustusDefaultValues(self):
        for key, item in self._allowed_options.items():
            if not self.get_value_or_none(key) and item.default_value and \
                    'augustus' in item.get_exclude() and \
                    'etraining' in item.get_exclude():
                if item.type == 'int':
                    self.set_value(key, int(item.default_value))
                elif item.type == 'bool':
                    self.set_value(key, bool(strtobool(item.default_value)))
                elif item.type == 'float':
                    self.set_value(key, float(item.default_value))
                else:
                    self.set_value(key, item.default_value)


def load_allowed_options(parameter_file, program=None):
    allowed_options = dict()
    with open(parameter_file, 'r') as file:
        options = json.load(file)

    for o in options:
        option = AugustusOption(o.get('name'), o.get('type'), o.get('possible_values'), o.get(
            'description'), o.get('usage'), o.get('default_value'), o.get('development'), o.get('exclude_apps'))

        if program == 'pygustus':
            if 'augustus' in option.get_exclude() and 'etraining' in option.get_exclude():
                allowed_options.update({option.get_name(): option})
        elif program:
            if not program in option.get_exclude():
                allowed_options.update({option.get_name(): option})
        else:
            allowed_options.update({option.get_name(): option})

    return allowed_options



import pytest
from pygustus import *
import os
import pygustus.util as util
import shutil


@pytest.mark.ghactions
def test_etraining_new_species():
    test_species_name = 'pygustus_test_spec'
    test_config_dir = 'tests/data/config'
    os.environ['AUGUSTUS_CONFIG_PATH'] = test_config_dir

    # remove test species if exists
    species_dir = os.path.join(test_config_dir, 'species', test_species_name)
    if os.path.exists(species_dir):
        shutil.rmtree(species_dir)

    # create species
    cmd = 'tests/data/scripts/new_species.pl'
    options = [f'--species={test_species_name}']
    util.execute_bin(cmd, options)

    # execute training for new species
    etraining.train('tests/data/genes.gb.train',
                    species=test_species_name, softmasking=False)


@pytest.mark.ghactions
def test_etraining_wrong_parameter():
    with pytest.raises(ValueError):
        etraining.train('tests/data/genes.gb.train',
                        species='pygustus_test_spec', smasking=False)


@pytest.mark.ghactions
def test_etraining_wrong_parameter_format():
    with pytest.raises(ValueError):
        etraining.train('tests/data/genes.gb.train',
                        species=True, softmasking=False)


@pytest.mark.ghactions
def test_etraining_wrong_bin_path():
    with pytest.raises(RuntimeError):
        etraining.train('tests/data/genes.gb.train', species='pygustus_test_spec',
                        softmasking=False, path_to_bin='/usr/local/bin')


@pytest.mark.ghactions
def test_etraining_wrong_bin_path_format():
    with pytest.raises(ValueError):
        etraining.train('tests/data/genes.gb.train', species='pygustus_test_spec',
                        softmasking=False, path_to_bin=42)



import os
import re


"""
Support to join results of several simultaneous runs of AUGUSTUS of
different sequence segments. Based on the AUGUSTUS script join_aug_pred.pl
"""
# TODO: gff3 support
# TODO: add gene droplist?


class Gene:
    def __init__(self, id, sequence, start, end, txt):
        self.id = id
        self.sequence = sequence
        self.start = start
        self.end = end
        self.txt = txt

    def __eq__(self, o: object) -> bool:
        return isinstance(o, Gene) and self.sequence == o.sequence and self.start == o.start

    def __ne__(self, o: object) -> bool:
        return not self == o

    def __str__(self) -> str:
        return f'Gene {self.id} starts at {self.start} and ends at {self.end} in sequence {self.sequence}.'

    def rename(self, id):
        old_id = self.id
        self.id = id
        self.txt = self.txt.replace(old_id, id)


class GFFFile:
    def __init__(self) -> None:
        self.header = None
        self.genes = list()

    def add_content(self, filepath):
        """Joins the given AUGUSTUS results.
        
        The files should be passed in the order of the AUGUSTUS runs.
        """
        if not os.path.isfile(filepath):
            raise ValueError(f'Could not open {filepath}')

        gene_txt = ''
        gene_collection = False
        with open(filepath) as file:
            line = file.readline()
            file_header = ''
            gff3 = False
            while line:
                # read file header
                if line.strip().startswith('##gff-version 3'):
                    gff3 = True

                if not self.header:
                    go_on1 = not 'prediction on sequence number' in line.strip()
                    go_on2 = not re.search(
                        "Looks like.*is in.*format", line.strip())
                    if go_on1 and go_on2:
                        file_header += line
                    else:
                        self.header = file_header

                # read genes
                # if back compatibility is required add condition like re.search("^### gene", line.strip())
                if re.search("^# start gene", line.strip()):
                    gene_collection = True

                if gene_collection:
                    gene_txt += line

                l_split = line.strip().split('\t')
                if len(l_split) > 5:
                    if l_split[2] == 'gene':
                        if gff3:
                            gid = l_split[-1].replace('ID=', '').split('.')[-1]
                        else:
                            gid = l_split[-1].split('.')[-1]
                        gseq = l_split[0]
                        gstart = l_split[3]
                        gend = l_split[4]

                # if back compatibility is required add condition like re.search("^### end gene", line.strip())
                if re.search("^# end gene", line.strip()):
                    gene = Gene(gid, gseq, gstart, gend, gene_txt)

                    # use unique gene name (id)
                    int_gid = int(gid.replace('g', ''))
                    if int_gid <= len(self.genes):
                        new_int_gid = len(self.genes) + 1
                        gene.rename(f'g{new_int_gid}')

                    # do not add redundant genes of two possibly overlapping neighboring runs
                    if len(self.genes) > 0:
                        if not gene in self.genes:
                            last_gene = self.genes[-1]
                            if gene.sequence == last_gene.sequence and int(gene.start) < int(last_gene.end):
                                pass
                            else:
                                self.genes.append(gene)
                        else:
                            last_gene = self.genes[-1]
                            if gene.sequence == last_gene.sequence and int(gene.start) == int(last_gene.start):
                                gene.rename(last_gene.id)
                                self.genes[-1] = gene
                    else:
                        self.genes.append(gene)

                    gene_collection = False
                    gene_txt = ''

                line = file.readline()

    def write(self, filename):
        with open(filename, "w") as file:
            file.write(self.header)
            for g in self.genes:
                file.write(g.txt)


def join_aug_pred(out_file, pred_files):
    """Joins the given AUGUSTUS results.
    
    After all result parts are joinedand it writes the result to the given
    out_file. The files should be passed in the order of the AUGUSTUS runs.

    Args:
        out_file (string): The path to the ouput file to write the
            joined results.
        pred_files (list): A list of AUGUSTUS result file names
            ordered by runs.
    """
    gff = GFFFile()
    for pred in pred_files:
        gff.add_content(pred)
    gff.write(out_file)


def create_hint_parts(inputfile, outfile, sequences, whitespaces=False):
    output = list()
    with open(inputfile) as file:
        line = file.readline()
        while line:
            if whitespaces:
                l_split = re.split(' +', line.strip())
            else:
                l_split = line.strip().split('\t')

            if len(l_split) > 1 and l_split[0] in sequences.keys():
                start, end = sequences[l_split[0]]
                if start > 0 and end > 0:
                    if int(l_split[3]) >= start and int(l_split[4]) <= end:
                        output.append(line)
                else:
                    output.append(line)

            line = file.readline()

    with open(outfile, "w") as file:
        for line in output:
            file.write(line)



#!/usr/bin/env python3

import sys
import os
import difflib


def compare_folder(reffolder, currentfolder, html=False, outputfolder='output_html/'):
    if not os.path.isdir(reffolder):
        return 'Reference folder not found: ' + reffolder
    if not os.path.isdir(currentfolder):
        return 'Current folder not found: ' + currentfolder

    res = ''

    for subdir, dirs, files in os.walk(reffolder):
        for file in files:
            currentdir = subdir.replace(reffolder, currentfolder)

            diff = compare_files(os.path.join(subdir, file),
                                 os.path.join(currentdir, file),
                                 html=html, outputfolder=outputfolder)
            res += diff

    return res


def compare_files(reffile, currentfile, html=False, outputfolder='output_html/'):
    if not os.path.isfile(reffile):
        return 'Reference file not found: ' + reffile
    if not os.path.isfile(currentfile):
        return 'Current file not found: ' + currentfile

    with open(reffile) as ff:
        reflines = ff.readlines()
    with open(currentfile) as tf:
        currentlines = tf.readlines()

    diff = ''.join(
        difflib.context_diff(reflines, currentlines, reffile, currentfile,
                             n=0))

    if html and diff:
        generate_html(reflines, currentlines, reffile,
                      currentfile, outputfolder)

    return diff


def generate_html(reflines, currentlines, reffile, currentfile, outputfolder):
    if not os.path.exists(outputfolder):
        os.mkdir(outputfolder)

    diff = difflib.HtmlDiff().make_file(reflines, currentlines, reffile,
                                        currentfile)

    filename = create_html_filename(reffile)
    with open(os.path.join(outputfolder, filename), 'w') as html:
        html.write(diff)


def create_html_filename(name):
    html_name = ''
    elements = []
    parts = name.split('/')

    for p in reversed(parts):
        elements.append(p)
        if 'test_' in p:
            break

    for e in reversed(elements):
        html_name += e + '_'

    return html_name + 'diff.html'


if __name__ == '__main__':
    reference = sys.argv[1]
    current = sys.argv[2]
    print(compare_files(reference, current))



# Keys for the JSON parameter file
NAME = 'name'
TYPE = 'type'
VALUES = 'possible_values'
DESCRIPTION = 'description'
USAGE = 'usage'
DEFAULT = 'default_value'
DEVELOPMENT = 'development'
EXCLUDE_APPS = 'exclude_apps' # possibilities see below

# Used types
TYPE_STRING = 'string'
TYPE_INT = 'int'
TYPE_FLOAT = 'float'
TYPE_BOOL = 'bool'
TYPE_LIST_STRING = 'list<string>'

# Exclude Apps
EXCLUDE_AUGUSTUS = 'augustus'
EXCLUDE_ETRAINING = 'etraining'
EXCLUDE_PYGUSTUS = 'pygustus'



#!/usr/bin/env python3

import json
import argparse
from constant import *

parser = argparse.ArgumentParser(
    description='Creates JSON file with all allowed AUGUSTUS parameters.')
parser.add_argument('-n', '--filename',
                    help='Desired file name for the JSON export. Default: parameters.json.')
args = parser.parse_args()

allowed_parameters = [
    {
        NAME: 'path_to_bin',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: 'path_to_binary=path/to/binary',
        DESCRIPTION: 'Sets the path to the desired executable version of AUGUSTUS when augustus.predict() is called or etraining when etraining.train() is called. The path is not saved for further executions.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'jobs',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: 'jobs=n',
        DEFAULT: '1',
        DESCRIPTION: 'If this option is set, AUGUSTUS is executed in parallel on sequence segments or split input files using n jobs. After the execution of all jobs, the output files are merged.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'chunksize',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: 'chunksize=n',
        DEFAULT: '2500000',
        DESCRIPTION: 'If this option is set and jobs > 1, each AUGUSTUS instance is executed on sequence segments of the maximum size n.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'overlap',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: 'overlap=n',
        DEFAULT: '500000',
        DESCRIPTION: 'If this option is set and jobs > 1, each AUGUSTUS instance is executed on sequence segments and the segments overlap by n.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'partitionHints',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: 'partitionHints=True/False',
        DEFAULT: 'False',
        DESCRIPTION: 'If this option is set to True, a hints file is given and jobs > 1, then the hints file is split into appropriate pieces for the respective AUGUSTUS jobs.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'minSplitSize',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: 'minSplitSize=n',
        DEFAULT: '0',
        DESCRIPTION: 'The input fasta file is spilt to at least n base pairs. Set this to 0 to split the input in single sequence files.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'maxSeqSize',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: 'maxSeqSize=n',
        DEFAULT: '3500000',
        DESCRIPTION: 'The maximum length of a sequence from which the sequence is started to be partitioned.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'partitionLargeSeqeunces',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: 'partitionLargeSeqeunces=True/False',
        DEFAULT: 'False',
        DESCRIPTION: 'Parallelize large sequences by automatically setting the AUGUSTUS parameters predictionStart and predictionEnd based on the given values for chunksize and overlap.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'debugOutputDir',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: 'debugOutputDir=path/to/dir',
        DEFAULT: None,
        DESCRIPTION: 'If the directory is specified, all generated files, i.e. the split of the input file and intermediate results, as well as the generated AUGUSTUS command lines are stored there. This option works only for the parallelization, i. e. jobs > 1 is set.',
        EXCLUDE_APPS: [EXCLUDE_AUGUSTUS, EXCLUDE_ETRAINING]
    },
    {
        NAME: 'species',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--species=SPECIES queryfilename',
        DESCRIPTION: 'The "queryfilename" is the filename (including relative path) to the file containing the query sequence(s) and SPECIES is an identifier for the species. Use --species=help to see a list in fasta format.'
    },
    {
        NAME: 'strand',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--strand=both/forward/backward',
        VALUES: ['both', 'forward', 'backward'],
        DEFAULT: 'both',
        DESCRIPTION: 'Predict genes on both strands, just the forward or just the backward strand. This effects the algorithm and for most cases "both" is recommended, even if genes on the unwanted strand then need to be ignored after they are predicted.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'genemodel',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--genemodel=partial/intronless/complete/atleastone/exactlyone',
        VALUES: ['partial', 'intronless', 'complete', 'atleastone', 'exactlyone'],
        DEFAULT: 'partial',
        DESCRIPTION:
            {'partial': 'allow prediction of incomplete genes at the sequence boundaries (default)',
             'intronless': 'only predict single-exon genes like in prokaryotes and some eukaryotes',
             'complete': 'only predict complete genes',
             'atleastone': 'predict at least one complete gene',
             'exactlyone': 'predict exactly one complete gene'},
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'UTR',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--UTR=true/false',
        DEFAULT: 'False',
        DESCRIPTION: 'Predict the untranslated regions in addition to the coding sequence. This works only for a subset of species for which an UTR model was trained.'
    },
    {
        NAME: 'singlestrand',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--singlestrand=true/false',
        DEFAULT: 'False',
        DESCRIPTION: 'Predict genes independently on each strand, allow overlapping genes on opposite strands. This is less accurate on average.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'hintsfile',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--hintsfile=hintsfilename',
        DESCRIPTION: 'When this option is used the prediction uses hints (= extrinsic information). The hintsfile contains the hints in GFF format.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'AUGUSTUS_CONFIG_PATH',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--AUGUSTUS_CONFIG_PATH=path',
        DESCRIPTION: 'Path to config directory (overrides environment variable $AUGUSTUS_CONFIG_PATH).',
    },
    {
        NAME: 'alternatives-from-evidence',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--alternatives-from-evidence=true/false',
        DESCRIPTION: 'Report alternative transcripts when they are suggested by hints. AUGUSTUS can then find alternative splice forms when the extrinsic evidence is not reconcilable with constitutive splicing only (1 tx per gene) and no overlaps.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'alternatives-from-sampling',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--alternatives-from-sampling=true/false',
        DESCRIPTION: 'Report alternative transcripts generated through probabilistic sampling. This can be useful to produce suboptimal but still plausible alternative gene structures.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'sample',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: '--sample=n',
        DEFAULT: '100',
        DESCRIPTION: 'The number of sampling iterations. The higher "n" is the more accurate is the estimation of probabilities (=scores output in 6th GFF column) but it usually is not important that the posterior probability is very accurate.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'minexonintronprob',
        DEVELOPMENT: False,
        TYPE: TYPE_FLOAT,
        USAGE: '--minexonintronprob=p',
        DEFAULT: '0.0',
        DESCRIPTION: 'The posterior probability of every exon and every intron in a transcript must be at least "minexonintronprob", otherwise the transcript is not reported. Value between 0 and 1 (default 0.0).',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]        
    },
    {
        NAME: 'minmeanexonintronprob',
        DEVELOPMENT: False,
        TYPE: TYPE_FLOAT,
        USAGE: '--minmeanexonintronprob=p',
        DEFAULT: '0.0',
        DESCRIPTION: 'The geometric mean of the probabilities of all exons and introns must be at least "minmeanexonintronprob". A value of 0.4 is reasonable if a restriction of results is desired at all. Value between 0 and 1 (default 0.0).',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'maxtracks',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: '--maxtracks=n',
        DEFAULT: '-1',
        DESCRIPTION: 'The maximum number of tracks when displayed in a genome browser is "maxtracks" (unless maxtracks=-1, then it is unbounded). In cases where all transcripts of a gene overlap at some position this is also the maximal number of transcripts for that gene. We recommend increasing the parameter "maxtracks" for improving sensitivity and setting "maxtracks" to 1 and increasing  minmeanexonintronprob and/or minexonintronprob in order to improve specificity. (default -1)',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'proteinprofile',
        DEVELOPMENT: False,
        TYPE: TYPE_STRING,
        USAGE: '--proteinprofile=filename',
        DESCRIPTION: 'Use the provided protein profile in the Protein-Profile-eXtension (PPX). See PPX section of RUNNING-AUGUSTUS.md.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'progress',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--progress=true',
        DEFAULT: 'False',
        DESCRIPTION: 'Show a progressmeter.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'gff3',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--gff3=true/false',
        DEFAULT: 'False',
        DESCRIPTION: 'Output in gff3 format.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'predictionStart',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: '--predictionStart=A',
        DESCRIPTION: 'Start prediction at sequence coordinate A. Default: 1. The output coordinates are nevertheless with respect to the sequence start. This is used, for example, to run augustus in parallel on overlappinging tiles of a large chromosome. ',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'predictionEnd',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: '--predictionEnd=B',
        DESCRIPTION: 'End prediction at sequence coodinate B. Default: sequence length. Feature for pros: If predictionEnd = predictionStart < 0  augustus uses the whole input sequence and just left-shift hints coordinates by B. This is intended for applications in which a sequence fragment is cut out of a chromosome for input to augustus but the hints have still original coordinates.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'noInFrameStop',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--noInFrameStop=true/false',
        DEFAULT: 'False',
        DESCRIPTION: 'Do not report transcripts with in-frame stop codons. Otherwise, intron-spanning stop codons could occur. Such transcripts are then removed from the output.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'noprediction',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--noprediction=true/false',
        DESCRIPTION: 'No gene predition is performed. Useful for getting the annotated protein sequences if the input is in Genbank format.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'uniqueGeneId',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--uniqueGeneId=true/false',
        DEFAULT: 'False',
        DESCRIPTION: 'If true, output gene identifyers like this: seqname.gN. This can be useful to avoid gene name conflicts when parallelizing.',
        EXCLUDE_APPS: [EXCLUDE_ETRAINING]
    },
    {
        NAME: 'softmasking',
        DEVELOPMENT: False,
        TYPE: TYPE_BOOL,
        USAGE: '--softmasking=True/False',
        DEFAULT: 'True',
        DESCRIPTION: 'Softmasked regions are treated as nonexonpart hints. As a consequence, exon overlapping the masked regions are discouraged. If the bases in repeat regions are lower case (a,c,g,t instead of A,C,G,T) in the input, then softmasking should be turned on. Softmasking is on average more accurate then hard-masking with Ns.'
    },
    {
        NAME: 'allow_hinted_splicesites',
        DEVELOPMENT: False,
        TYPE: TYPE_LIST_STRING,
        USAGE: '--allow_hinted_splicesites=atac,...',
        DESCRIPTION: 'Allows other non-standard splice sites, such as at-ac. The given list is appended to the list of standard splice site dinucleotide consensus pairs: gtag,gcag'
    },
    {
        NAME: 'alnfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/augustus/verbosity',
        DEVELOPMENT: False,
        TYPE: TYPE_INT,
        USAGE: '--/augustus/verbosity=n',
        DEFAULT: '1',
        DESCRIPTION: 'Value of 0, 1, 2 or 3 produce increasingly verbose output.'
    },
    {
        NAME: '/BaseCount/weighingType',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/BaseCount/weightMatrixFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'bridge_genicpart_bonus',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'canCauseAltSplice',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'capthresh',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'cds',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'checkExAcc',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'codingseq',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'codonAlignmentFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'complete_genes',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/assmotifqthresh',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/assqthresh',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/dssqthresh',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/compSigScoring',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/conservation',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/covPen',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ec_score',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_gain',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_loss',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ali_error',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/computeNumSubs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/maxIterations',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/mil_factor',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ec_factor',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ec_addend',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ec_thold',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/ic_thold',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/genesWithoutUTRs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/liftover_all_ECs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/logreg',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/maxCov',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/omega',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/num_omega',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/num_features',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/scale_codontree',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/oeExtensionWidth',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/onlySampledECs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/only_species',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/outdir_orthoexons',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/outdir',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/printOrthoExonAli',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/printExonCandsMSA',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/printConservationWig',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/phylo_factor',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/phylo_model',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/dd_factor',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/dd_rounds',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/dd_step_rule',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/dualdecomp',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/overlapcomp',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/lambda',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/almost_identical_maxdiff',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/amberprob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/ass_end',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/ass_maxbinsize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/ass_start',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/ass_upwindow_size',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/decomp_num_at',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/decomp_num_gc',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/decomp_num_steps',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/dss_end',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/dss_maxbinsize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/dss_start',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/gc_range_max',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/gc_range_min',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/init_coding_len',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/intterm_coding_len',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/max_contra_supp_ratio',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/min_coding_len',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/ochreprob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/opalprob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/probNinCoding',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/subopt_transcript_threshold',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/tis_maxbinsize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/trans_init_window',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Constant/tss_upwindow_size',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'contentmodels',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRF',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRF_N',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRF_TRAIN',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainCDS',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainIntron',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainIgenic',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainSS',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainUTR',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'CRFtrainTIS',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'dbaccess',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'dbhints',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/state00',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/state01',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/state02',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/state03',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/statecount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/EHMMTraining/trainfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'emiprobs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'errfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'evalset',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'exoncands',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/etorder',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/etpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/exonlengthD',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/infile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/k',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/lenboostE',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/lenboostL',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/maxexonlength',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/minexonlength',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/minPatSum',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/minwindowcount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/patpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/slope_of_bandwidth',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/tisalpha',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/tis_motif_memory',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/tis_motif_radius',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ExonModel/verbosity',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'exonnames',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'extrinsicCfgFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'GCwinsize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'GD_stepsize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/genbank/verbosity',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/HMMTraining/savefile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IGenicModel/infile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IGenicModel/k',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IGenicModel/outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IGenicModel/patpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IGenicModel/verbosity',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/allow_dss_consensus_gc',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/ass_motif_memory',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/ass_motif_radius',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/asspseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/d',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/dssneighborfactor',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/dsspseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/infile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/k',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/minwindowcount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/non_ag_ass_prob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/non_gt_dss_prob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/patpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/sf_with_motif',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/slope_of_bandwidth',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/splicefile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/ssalpha',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/IntronModel/verbosity',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'introns',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'keep_viterbi',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'label_flip_prob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'learning_rate',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'lossweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'locustree',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'maxDNAPieceSize',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'maxOvlp',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'max_sgd_inter',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'mea',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'mea_evaluation',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/no_compatible_edges',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/alpha_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/alpha_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/x0_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/x0_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/x1_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/x1_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/y0_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/y0_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/i1_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/i1_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/i2_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/i2_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/j1_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/j1_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/j2_E',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/j2_I',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/weight_base',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/r_be',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/r_bi',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/weight_exon',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/weight_gene',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/MeaPrediction/weight_utr',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'min_intron_len',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'optCfgFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printEvidence',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printHints',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printMEA',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printOEs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printSampled',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printGeneRangesBED',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'printGeneRangesGFF',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'param_outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'print_blocks',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'print_utr',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'protein',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/allow_truncated',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/exhaustive_substates',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/block_threshold_spec',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/block_threshold_sens',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/blockpart_threshold_spec',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/blockpart_threshold_sens',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/global_factor_threshold',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/absolute_malus_threshold',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/invalid_score',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/ProteinModel/weight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'referenceFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'refSpecies',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'rescaleBoni',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'rLogReg',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'scorediffweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'speciesfilenames',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'start',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'stop',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'stopCodonExcludedFromCDS',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'temperature',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'tieIgenicIntron',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'trainFeatureFile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'translation_table',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'treefile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'truncateMaskedUTRs',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'tss',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'tts',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'uniqueCDS',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'useAminoAcidRates',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'useNonCodingModel',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'use_sgd',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'useTFprob',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/d_polya_cleavage_max',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/d_polya_cleavage_min',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/d_polyasig_cleavage',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/d_tss_tata_max',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/d_tss_tata_min',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/exonlengthD',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/infile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/k',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/max3singlelength',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/max3termlength',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/maxexonlength',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/minwindowcount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/outfile',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/patpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/prob_polya',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/slope_of_bandwidth',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tata_end',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tata_pseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tata_start',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tss_end',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tss_start',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tssup_k',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tssup_patpseudocount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/tts_motif_memory',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/utr3patternweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/utr5patternweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/utr3prepatternweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/utr5prepatternweight',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/UtrModel/verbosity',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Testing/testMode',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/Testing/workingDir',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score0',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score1',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score2',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score3',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score4',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score5',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score6',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score7',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score8',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score9',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score10',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score11',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score12',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score13',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score14',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score15',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score16',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score17',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score18',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score19',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score20',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score21',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score22',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score23',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score24',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score25',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score26',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score27',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score28',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score29',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score30',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score31',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score32',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score33',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score34',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score35',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/exon_score36',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/intron_score0',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/intron_score1',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/intron_score2',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/CompPred/intron_score3',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'lg_exon_score0',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'lg_exon_score1',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'lg_exon_score2',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: 'lg_exon_score3',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/NAMGene/statecount',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    },
    {
        NAME: '/NAMGene/SynchState',
        DEVELOPMENT: True,
        DESCRIPTION: ''
    }
]


def export(filename='parameters.json'):
    with open(filename, 'w') as file:
        json.dump(allowed_parameters, file, indent=4, sort_keys=False)


def sort_key(item):
    return item[NAME].lower()

if __name__ == '__main__':
    allowed_parameters.sort(key=sort_key)
    if args.filename is not None:
        export(args.filename)
    else:
        export()



"""
A python wrapper for the gene prediction program AUGUSTUS.
"""

from pkg_resources import resource_filename
from pygustus.options import aug_options
import pygustus.util as util
import pygustus.fasta_methods as fm
import gzip
import os
import textwrap

__all__ = ['predict', 'config_get_bin',
           'config_set_bin', 'config_set_default_bin', 'show_fasta_info',
           'show_aug_help', 'show_aug_paramlist', 'show_species_info', 'help']


PARAMETER_FILE = resource_filename('pygustus.options', 'parameters.json')
MIN_AUG_VERSION = '3.3.2'


def predict(*args, options=None, **kwargs):
    """Executes the binary of AUGUSTUS.
    
    For the execution the given parameters are passed as
    command line arguments.

    Args:
        *args (tuple): Exactly one argument should be passed here.
            Either the queryfilename or one of the help calls of
            AUGUSTUS (--help, --pramlist).
        options (AugustusOptions): Optional; If an instance of AugustusOptions
            is passed, it will be used for the call. Otherwise, a new instance
            is created based on the passed arguments (the default is None).
        **kwargs (dict): Arguments for AUGUSTUS or Pygustus as dict: lists with
            possible parameters can be obtained from the help methods or 
            the Pygustus README (only Pygustus parameters).
    """

    pygustus_options = util.get_options(
        *args, options=options, path_to_params=PARAMETER_FILE, program='pygustus', **kwargs)

    augustus_command = config_get_bin()
    tmp_path_to_bin = util.get_path_to_binary(pygustus_options, 'AUGUSTUS')
    if tmp_path_to_bin:
        augustus_command = tmp_path_to_bin

    util.check_bin(augustus_command)
    util.check_aug_version(augustus_command, MIN_AUG_VERSION)

    aug_options = util.get_options(
        *args, options=options, path_to_params=PARAMETER_FILE, program='augustus', **kwargs)

    jobs = pygustus_options.get_value_or_none('jobs')
    chunksize = pygustus_options.get_value_or_none('chunksize')
    overlap = pygustus_options.get_value_or_none('overlap')
    partition_hints = pygustus_options.get_value_or_none('partitionHints')
    minsize = pygustus_options.get_value_or_none('minSplitSize')
    partition_sequences = pygustus_options.get_value_or_none(
        'partitionLargeSeqeunces')
    debug_dir = pygustus_options.get_value_or_none('debugOutputDir')
    max_seq_size = pygustus_options.get_value_or_none('maxSeqSize')

    # check input file
    zip = False
    is_set, input_file = aug_options.get_input_filename()
    if is_set:
        if input_file:
            util.check_file(input_file)
        else:
            raise ValueError(f'Input file not specified.')

        # unzip if gz file is given
        f_name, f_ext = os.path.splitext(input_file)
        if f_ext == '.gz':
            nf = open(f_name, 'wb')
            with gzip.open(input_file) as f:
                bindata = f.read()
                nf.write(bindata)
            nf.close()
            aug_options.set_input_filename(f_name)
            zip = True

    if jobs and jobs > 1:
        util.execute_bin_parallel(
            augustus_command, aug_options, jobs, chunksize, overlap, partition_sequences, partition_hints, minsize, max_seq_size, debug_dir)
    else:
        print(f'Execute AUGUSTUS with given options.')

        util.execute_bin(augustus_command, aug_options.get_options())
        
        outfile = aug_options.get_value_or_none('outfile')
        if outfile:
            print(f'Output written to: {outfile}')

    if zip:
        os.remove(f_name)


def config_get_bin():
    """Returns currently configured path to the executable of AUGUSTUS.

    Returns:
        string: The currently configured path to the executable of AUGUSTUS.
    """
    return util.get_config_item('augustus_bin')


def config_set_bin(value):
    """Updates the configured path to the executable of AUGUSTUS.

    Args:
        value (string): The path to the execuatble of AUGUSTUS as string.

    Raises:
        RuntimeError: If the given path does not exist or the file
        is not executable.
    """
    util.check_bin(value)
    util.set_config_item('augustus_bin', value)


def config_set_default_bin():
    """Sets the configured path to the AUGUSTUS executable to 'augustus'.

    This should exist if AUGUSTUS is properly installed on the system.
    """
    util.set_config_item('augustus_bin', 'augustus')


def show_fasta_info(inputfile):
    """Outputs information about a fasta file.
    
    This method outputs information about the contents of the passed file
    in fasta format. It is based on the Pearl script summarizeACGTcontent.pl.

    Args:
        inputfile (string): Path to the file in fasta format as string.
    """
    fm.summarize_acgt_content(inputfile)


def show_aug_help():
    """Shows the help output of AUGUSTUS.
    """
    predict('--help')


def show_aug_paramlist():
    """Shows possible parameter names of AUGUSTUS.
    """
    predict('--paramlist')


def show_species_info():
    """Shows species information of AUGUSTUS.
    """
    predict('--species=help')


def help():
    """Shows usage information.
    """

    help_msg = """usage:
augustus.predict(queryfilename, species='SPECIES', [augustus_parameters], [pygustus_parameters])

'queryfilename' is the filename (including relative path) to the file
containing the query sequence(s) in fasta format.

SPECIES is an identifier for the species.
Use augustus.show_species_info() to see a list.

augustus_parameters are all possible parameters for AUGUSTUS.
Use augustus.show_aug_help() to find more information or
augustus.show_aug_paramlist() to see a list.

pygustus_parameters:

"""
    pygustus_options = aug_options.load_allowed_options(
        PARAMETER_FILE, program='pygustus')
    for opt in pygustus_options.values():
        help_msg += f'{opt.name} ({opt.type})\n'
        if opt.description:
            lines = textwrap.wrap(opt.description)
            help_msg += '  description:\n'
            for l in lines:
                help_msg += f'    {l} \n'
        if opt.possible_values:
            help_msg += f'  possible values: {opt.possible_values} \n'
        if opt.default_value:
            help_msg += f'  default value: {opt.default_value} \n'
        help_msg += '\n'

    print(help_msg.rstrip())



#!/usr/bin/env python3

import os
import sys


def pred(sourcepath, targetpath):
    startwith = '# start gene'
    ignores = [
        '# ----- prediction', '###',
        '# command line', '# augustus',
        '# Predicted genes for sequence']
    ignoredlines = search_for_lines_to_ignore(sourcepath, startwith, ignores)
    filter(sourcepath, targetpath, ignoredlines)


def search_for_lines_to_ignore(sourcepath, startwith, ignores, startblock=True):
    ignoredlines = []

    # startblock
    if startblock:
        with open(sourcepath) as fp:
            line = fp.readline()
            lno = 1
            while line:
                if startwith in line.strip():
                    break
                line = fp.readline()
                ignoredlines.append(lno)
                lno += 1

    # other lines to ignore
    with open(sourcepath) as fp:
        line = fp.readline()
        lno = 1
        while line:
            if len(ignores) > 0 and any(s in line.strip() for s in ignores):
                ignoredlines.append(lno)

            if line.startswith('#\n'):
                ignoredlines.append(lno)

            line = fp.readline()
            lno += 1

    return ignoredlines


def filter(sourcepath, targetpath, ignoredlines):
    if os.path.exists(targetpath):
        os.remove(targetpath)

    with open(targetpath, "w") as file_out:
        with open(sourcepath) as fp:
            line = fp.readline()
            lno = 1
            while line:
                if lno not in ignoredlines:
                    file_out.write(line.strip() + '\n')
                line = fp.readline()
                lno += 1


if __name__ == '__main__':
    source = sys.argv[1]
    target = sys.argv[2]
    pred(source, target)



from Bio import SeqIO
import pygustus.util as util


def summarize_acgt_content(inputfile):
    util.check_file(inputfile)

    letters = ['a', 'c', 'g', 't', 'n']
    file_sum = dict.fromkeys(letters, 0)
    file_sum.update({'rest': 0})
    seq_count = 0

    for seq_record in SeqIO.parse(inputfile, 'fasta'):
        seq_count += 1
        seq_sum = 0
        print_seq_acgt = ''

        for l in letters:
            value = seq_record.seq.lower().count(l)
            seq_sum += value
            if l != 'n':
                print_seq_acgt += f'   {value} {l}'
            else:
                if value > 0:
                    print_seq_acgt += f'   {value} {l}'

            update_values(file_sum, l, value)

        rest = len(seq_record) - seq_sum
        if rest > 0:
            print_seq_acgt += f'   {rest} ?'
            update_values(file_sum, 'rest', rest)

        print_seq_line = f'{len(seq_record)} bases.\t{seq_record.id} BASE COUNT  {print_seq_acgt}'
        print(print_seq_line)

    summary_acgt = ''
    complete_bp = 0
    sum_acgt = 0

    for l in letters:
        if l != 'n':
            summary_acgt += f'   {file_sum[l]} {l}'
            complete_bp += file_sum[l]

    sum_acgt += complete_bp

    if file_sum['n'] > 0:
        summary_acgt += f'   {file_sum[l]} {l}'
        complete_bp += file_sum['n']

    if file_sum['rest'] > 0:
        summary_acgt += f'   {file_sum[l]} {l}'
        complete_bp += file_sum['rest']

    gc = 100 * float(file_sum['g'] + file_sum['c']) / sum_acgt

    print(f'summary: BASE COUNT  {summary_acgt}')
    print(f'total {complete_bp}bp in {seq_count} sequence(s).')
    print(f'gc: {gc}%')


def update_values(file_sum, key, value):
    cur_value = file_sum[key]
    file_sum.update({key: cur_value + value})


def split(inputfile, outputdir, chunksize, overlap, partition_sequences, minsize, max_seq_size):
    util.check_file(inputfile)
    util.rmtree_if_exists(outputdir, even_none_empty=True)
    util.mkdir_if_not_exists(outputdir)

    fileidx = 0
    run = 0
    filesize = 0
    records_to_write = list()
    records = list(SeqIO.parse(inputfile, 'fasta'))
    run_information = list()

    for seq_record in records:
        seqsize = len(seq_record)

        if seqsize > max_seq_size:
            if len(records_to_write) > 0:
                fileidx += 1
                run += 1
                run_information.append(
                    {
                        'run': run,
                        'fileidx': fileidx,
                        'seqinfo': {x.id: [0, 0] for x in records_to_write}
                    })
                write_file(records_to_write, inputfile, outputdir, fileidx)
                filesize = 0

            fileidx += 1
            write_file([seq_record], inputfile, outputdir, fileidx)
            if partition_sequences:
                if chunksize == 0:
                    chunksize = 2500000
                if chunksize > 3500000:
                    chunksize = 3500000
                if overlap == 0:
                    overlap = int(chunksize / 6)
                chunks = list()
                go_on = True
                while go_on:
                    if len(chunks) == 0:
                        chunks.append([1, chunksize])
                    else:
                        last_start, last_end = chunks[-1]
                        start = last_end + 1 - overlap
                        end = start + chunksize - 1
                        if end >= seqsize:
                            end = seqsize
                            go_on = False
                        chunks.append([start, end])
                for c in chunks:
                    run += 1
                    run_information.append(
                        {
                            'run': run,
                            'fileidx': fileidx,
                            'seqinfo': {seq_record.id: [c[0], c[1]]}
                        })
            else:
                run += 1
                run_information.append(
                    {
                        'run': run,
                        'fileidx': fileidx,
                        'seqinfo': {seq_record.id: [0, 0]}
                    })
        elif minsize == 0 or filesize + seqsize >= minsize or seq_record.id == records[-1].id:
            records_to_write.append(seq_record)
            fileidx += 1
            run += 1
            run_information.append(
                {
                    'run': run,
                    'fileidx': fileidx,
                    'seqinfo': {x.id: [0, 0] for x in records_to_write}
                })
            write_file(records_to_write, inputfile, outputdir, fileidx)
            filesize = 0
        else:
            records_to_write.append(seq_record)
            filesize += seqsize

    return run_information


def write_file(records_to_write, inputfile, outputdir, fileidx):
    splitpath = util.create_split_filenanme(
        inputfile, outputdir, fileidx)
    SeqIO.write(records_to_write, splitpath, 'fasta')
    records_to_write.clear()


def get_sequence_count(inputfile):
    util.check_file(inputfile)
    sequences = list(SeqIO.parse(inputfile, 'fasta'))
    return len(sequences)


def get_sequence_size(inputfile, idx=0):
    util.check_file(inputfile)
    sequences = list(SeqIO.parse(inputfile, 'fasta'))
    return len(sequences[idx])


def get_sequence_id(inputfile, idx=0):
    util.check_file(inputfile)
    sequences = list(SeqIO.parse(inputfile, 'fasta'))
    return sequences[idx].id



#!/usr/bin/env python

from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pygustus',
    version='0.6.0',
    description='Python wrapper for AUGUSTUS.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Daniel Honsel',
    author_email='dhonsel@cs.uni.goettingen.de',
    url='https://github.com/Gaius-Augustus/pygustus',
    download_url='https://github.com/Gaius-Augustus/pygustus/zipball/master',
    packages=find_packages(),
    package_data={'pygustus': ['options/parameters.json', 'config.json']},
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    install_requires=[
        "biopython",
    ],
)



import pytest
import os
import shutil
from pygustus import *
import pygustus.aug_out_filter as afilter
import pygustus.aug_comparator as comp
import wget
import tarfile

# TODO: [GH-Actions] upload generated html diff??


@pytest.mark.ghactions
def test_augustus_help():
    augustus.predict('--help')


@pytest.mark.ghactions
def test_augustus_simple():
    augustus.predict('tests/data/example.fa', species='human',
                     UTR=True, softmasking=False)


@pytest.mark.ghactions
def test_augustus_wrong_parameter():
    with pytest.raises(ValueError):
        augustus.predict('tests/data/example.fa', species='human',
                         UTR=True, smasking=False)


@pytest.mark.ghactions
def test_augustus_wrong_parameter():
    with pytest.raises(ValueError):
        augustus.predict('tests/data/example.fa', species='human',
                         UTR=True, softmasking=0.42)


@pytest.mark.ghactions
def test_augustus_wrong_bin_path():
    with pytest.raises(RuntimeError):
        augustus.predict('tests/data/example.fa', species='human',
                         UTR=True, softmasking=False, path_to_bin='/usr/local/bin')


@pytest.mark.ghactions
def test_augustus_wrong_bin_path_format():
    with pytest.raises(ValueError):
        augustus.predict('tests/data/example.fa', species='human',
                         UTR=True, softmasking=False, path_to_bin=False)


@pytest.mark.ghactions
def test_augustus_parallel_two_jobs_and_sequences():
    name = test_augustus_parallel_two_jobs_and_sequences.__name__
    options = {'species': 'human', 'UTR': True, 'softmasking': False}

    run_parallel_tests('tests/data/example.fa', jobs=2, testname=name,
                       options=options)


@pytest.mark.ghactions
def test_augustus_parallel_with_hints():
    name = test_augustus_parallel_with_hints.__name__
    options = {'species': 'human', 'UTR': True, 'softmasking': False,
               'hintsfile': 'tests/data/hints.gff',
               'extrinsicCfgFile': 'tests/data/config/extrinsic/extrinsic.MPE.cfg'}

    run_parallel_tests('tests/data/example.fa', jobs=2, testname=name,
                       options=options)


@pytest.mark.ghactions
def test_augustus_parallel_two_jobs_unique_geneid():
    name = test_augustus_parallel_two_jobs_unique_geneid.__name__
    options = {'species': 'human', 'UTR': True, 'softmasking': False,
               'uniqueGeneId': True}

    run_parallel_tests('tests/data/example.fa', jobs=2, testname=name,
                       options=options)


@pytest.mark.ghactions
def test_augustus_parallel_two_jobs_and_sequences_gff3():
    name = test_augustus_parallel_two_jobs_and_sequences_gff3.__name__
    options = {'species': 'human', 'UTR': True, 'softmasking': False,
               'gff3': True}

    run_parallel_tests('tests/data/example.fa', jobs=2, testname=name,
                       options=options)


@pytest.mark.ghactions
def test_augustus_parallel_two_jobs_gff3_and_geneid():
    name = test_augustus_parallel_two_jobs_gff3_and_geneid.__name__
    options = {'species': 'human', 'UTR': True, 'softmasking': False,
               'gff3': True, 'uniqueGeneId': True}

    run_parallel_tests('tests/data/example.fa', jobs=2, testname=name,
                       options=options)


@pytest.mark.ghactions
def test_augustus_parallel_one_large_sequence():
    name = test_augustus_parallel_one_large_sequence.__name__
    options = {
        'species': 'human',
        'UTR': True,
        'softmasking': True,
        'partitionLargeSeqeunces': True,
        'chunksize': 250000,
        'overlap': 50000,
        'maxSeqSize': 750000
        }

    run_parallel_tests('tests/data/genome.fa', jobs=5, testname=name,
                       options=options)


def run_parallel_tests(inputfile, jobs, testname, options):
    outdir = os.path.join('tests/out', testname)
    out_html = os.path.join(outdir, 'output_html')
    out_augustus_tmp = os.path.join(outdir, 'augustus_tmp.gff')
    out_augustus_joined_tmp = os.path.join(outdir, 'augustus_joined_tmp.gff')
    out_augustus = os.path.join(outdir, 'augustus.gff')
    out_augustus_joined = os.path.join(outdir, 'augustus_joined.gff')

    if (os.path.exists(outdir)):
        shutil.rmtree(outdir)
    os.makedirs(outdir)

    assert os.path.exists(outdir)

    # run augustus
    augustus.predict(inputfile, **options, outfile=out_augustus_tmp)
    assert os.path.exists(out_augustus_tmp)

    # run multiple augustus jobs on the same input file
    augustus.predict(inputfile, **options,
                     outfile=out_augustus_joined_tmp, jobs=jobs)
    assert os.path.exists(out_augustus_joined_tmp)

    # filter both results
    afilter.pred(out_augustus_tmp, out_augustus)
    afilter.pred(out_augustus_joined_tmp, out_augustus_joined)
    os.remove(out_augustus_tmp)
    os.remove(out_augustus_joined_tmp)

    # compare results
    diff = comp.compare_files(
        out_augustus, out_augustus_joined, html=True, outputfolder=out_html)
    assert diff == ''


@pytest.mark.expensive
def test_augustus_parallel_large_sequence_hints():
    outdir = os.path.join(
        'tests/out', test_augustus_parallel_large_sequence_hints.__name__)
    out_html = os.path.join(outdir, 'output_html')
    out_augustus_tmp = os.path.join(
        'tests/data/chr2L/reference', 'aug.nasonia.hints.joined_tmp.gff')
    out_augustus_joined_tmp = os.path.join(outdir, 'augustus_joined_tmp.gff')
    out_augustus = os.path.join(outdir, 'aug.nasonia.hints.joined.gff')
    out_augustus_joined = os.path.join(outdir, 'augustus_joined.gff')
    options = {'species': 'nasonia', 'softmasking': True, 'chunksize': 3500000, 'overlap': 700000,
               'hintsfile': 'tests/data/chr2L/hints.gff', 'partitionLargeSeqeunces': True,
               'extrinsicCfgFile': 'tests/data/config/extrinsic/extrinsic.M.RM.E.W.cfg'}
    if (os.path.exists(outdir)):
        shutil.rmtree(outdir)
    os.makedirs(outdir)

    assert os.path.exists(outdir)

    # run multiple augustus jobs on the same input file
    augustus.predict('tests/data/chr2L/chr2L.sm.fa.gz', **options,
                     outfile=out_augustus_joined_tmp, jobs=9)
    assert os.path.exists(out_augustus_joined_tmp)

    # filter both results
    afilter.pred(out_augustus_tmp, out_augustus)
    afilter.pred(out_augustus_joined_tmp, out_augustus_joined)
    os.remove(out_augustus_joined_tmp)

    # compare results
    diff = comp.compare_files(
        out_augustus, out_augustus_joined, html=True, outputfolder=out_html)
    assert diff == ''


@pytest.mark.expensive
def test_augustus_parallel_large_sequence_partition_hints():
    outdir = os.path.join(
        'tests/out', test_augustus_parallel_large_sequence_partition_hints.__name__)
    out_html = os.path.join(outdir, 'output_html')
    out_augustus_tmp = os.path.join(
        'tests/data/chr2L/reference', 'aug.nasonia.hints.part.joined_tmp.gff')
    out_augustus_joined_tmp = os.path.join(outdir, 'augustus_joined_tmp.gff')
    out_augustus = os.path.join(outdir, 'aug.nasonia.hints.joined.gff')
    out_augustus_joined = os.path.join(outdir, 'augustus_joined.gff')
    options = {'species': 'nasonia', 'softmasking': True, 'chunksize': 3500000, 'overlap': 700000,
               'hintsfile': 'tests/data/chr2L/hints.gff', 'partitionHints': True,
               'partitionLargeSeqeunces': True,
               'extrinsicCfgFile': 'tests/data/config/extrinsic/extrinsic.M.RM.E.W.cfg'}
    if (os.path.exists(outdir)):
        shutil.rmtree(outdir)
    os.makedirs(outdir)

    assert os.path.exists(outdir)

    # run multiple augustus jobs on the same input file
    augustus.predict('tests/data/chr2L/chr2L.sm.fa.gz', **options,
                     outfile=out_augustus_joined_tmp, jobs=9)
    assert os.path.exists(out_augustus_joined_tmp)

    # filter both results
    afilter.pred(out_augustus_tmp, out_augustus)
    afilter.pred(out_augustus_joined_tmp, out_augustus_joined)
    os.remove(out_augustus_joined_tmp)

    # compare results
    diff = comp.compare_files(
        out_augustus, out_augustus_joined, html=True, outputfolder=out_html)
    assert diff == ''


@pytest.mark.expensive
def test_chlamy_parallel():
    datadir = 'tests/data'
    testdir = os.path.join(datadir, 'chlamy')

    if not os.path.exists(testdir):
        data_url = 'http://augustus.uni-greifswald.de/bioinf/downloads/data/chlamy/chlamy.tgz'
        data_file = os.path.join(datadir, 'chlamy.tgz')
        wget.download(data_url, out=data_file)

        data_tar = tarfile.open(data_file)
        data_tar.extractall(datadir)
        data_tar.close()
        os.remove(data_file)

    outdir = os.path.join(
        'tests/out', test_chlamy_parallel.__name__)
    out_augustus_joined_tmp = os.path.join(outdir, 'augustus_joined_tmp.gff')
    if (os.path.exists(outdir)):
        shutil.rmtree(outdir)
    os.makedirs(outdir)

    options = {'species': 'chlamy2011', 'softmasking': True, 'partitionLargeSeqeunces': False,
               'minSplitSize': 1000000, 'hintsfile': 'tests/data/chlamy/hints.gff', 'UTR': True,
               'extrinsicCfgFile': 'tests/data/config/extrinsic/extrinsic.M.RM.E.W.cfg'}

    augustus.predict('tests/data/chlamy/genome.fa', **options,
                     outfile=out_augustus_joined_tmp, jobs=8)
